{ =============================================================
(c) 2000 Andrei Kashin (Andrei.Kashin@pobox.spbu.ru)

Version 1.00

================================================================}

unit URL_Loader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  URLDownload, ComObj;

type

TDownloadOption =
 (dlopt_NoProgressiveRendering,
  dlopt_OfflineOperation,
  dlopt_GetNewestVersion,
  dlopt_NoWriteCache,
  dlopt_NeedFile,
  dlopt_PullData,
  dlopt_IgnoreSecurityProblem,
  dlopt_Resynchronize,
  dlopt_HyperLink,
  dlopt_NoUI,
  dlopt_SilentOperation,
  dlopt_PragmaNoCache,
  dlopt_DirectRead,
  dlopt_FormsSubmit,
  dlopt_GetFromCacheIfNetFailed);

TDownloadOptions = set of TDownloadOption;

TURL_Loader = class;

TDownloadProgress_Notify =
 procedure (Sender: TURL_Loader; const Progress:TDownloadProgress) of object;
TBytesAvailable_Notify =
 procedure (Sender: TURL_Loader; BytesAvailable: integer;
            var BytesToRead: integer; var ReadBuffer: pointer) of object;
TDataRead_Notify =
 procedure (Sender: TURL_Loader; Data:pointer; DataSize: integer) of object;
TDownloadStop_Notify =
 procedure (Sender: TURL_Loader; ErrorCode: cardinal) of object;
THTTPRequestData_Notify =
 procedure (Sender: TURL_Loader; var Method: string; var ExtraHeaders:string;
            var RequestBody:string) of object;
THTTPResponse_Notify =
 procedure (Sender: TURL_Loader; ResponseCode: DWord;
            const ResponseHeaders: string) of object;
TDownloadToStream_Notify =
 procedure (Sender: TURL_Loader; ErrorCode: cardinal;
            Stream: TStream; StartPosition: integer) of object;

TURL_Loader = class(TComponent)
 private
  { Private declarations }
  FURLDownloader: TURLDownloader;
  FURL: string;
  FDownloadOptions: TDownloadOptions;
  FAXControl: TComObject;

  FOnDownloadProgress: TDownloadProgress_Notify;
  FOnBytesAvailable: TBytesAvailable_Notify;
  FOnDataRead: TDataRead_Notify;
  FOnDownloadStop: TDownloadStop_Notify;
  FOnHTTPRequestData: THTTPRequestData_Notify;
  FOnHTTPResponse: THTTPResponse_Notify;
  FOnDownloadToStream: TDownloadToStream_Notify;

  procedure SetURL(const Value:string);
  function GetDownloadInProgress:boolean;
  function GetTotalRead: integer;
  procedure StartDownloadCommon(ToStream:boolean; Stream:TStream);

  procedure DownloadProgressNotify(Sender: TURLDownloader; const Progress:TDownloadProgress);
  procedure BytesAvailableNotify(Sender: TURLDownloader; BytesAvailable: integer;
                                 var BytesToRead: integer; var ReadBuffer: pointer);
  procedure DataReadNotify(Sender: TURLDownloader; Data:pointer; DataSize: integer);
  procedure DownloadStopNotify(Sender: TURLDownloader; ErrorCode: cardinal);
  procedure HTTPResponseNotify(Sender: TURLDownloader; ResponseCode: DWord;
                               const ResponseHeaders: string);
  procedure DownloadToStreamNotify(Sender: TDownloaderToStream; ErrorCode: cardinal;
                                   Stream: TStream; StartPosition: integer);
 public
  { Public declarations }
  constructor Create(AOwner: TComponent);override;
  destructor Destroy; override;

  procedure StartDownload;
  procedure StartDownloadToStream(Stream:TStream);
  procedure AbortDownload;

  property AXControl: TComObject read FAXControl write FAXControl;
  property DownloadInProgress: boolean read GetDownloadInProgress;
  property TotalRead: integer read GetTotalRead;
 published
  { Published declarations }
  property URL:string read FURL write SetURL;
  property DownloadOptions: TDownloadOptions
            read FDownloadOptions write FDownloadOptions;

  {Events}
  property OnDownloadProgress: TDownloadProgress_Notify
            read FOnDownloadProgress write FOnDownloadProgress;
  property OnBytesAvailable: TBytesAvailable_Notify
            read FOnBytesAvailable write FOnBytesAvailable;
  property OnDataRead: TDataRead_Notify
            read FOnDataRead write FOnDataRead;
  property OnDownloadStop: TDownloadStop_Notify
            read FOnDownloadStop write FOnDownloadStop;
  property OnHTTPRequestData: THTTPRequestData_Notify
            read FOnHTTPRequestData write FOnHTTPRequestData;
  property OnHTTPResponse: THTTPResponse_Notify
            read FOnHTTPResponse write FOnHTTPResponse;
  property OnDownloadToStream: TDownloadToStream_Notify
            read FOnDownloadToStream write FOnDownloadToStream;
end;

procedure Register;

implementation

uses UrlMon, AxCtrls;

procedure Register;
begin
  RegisterComponents('Internet', [TURL_Loader]);
end;

function DownloadOptionsToBindFlags(DownloadOptions:TDownloadOptions):DWord;
begin
 Result := 0;
 if dlopt_NoProgressiveRendering in DownloadOptions then
  Result := Result or BINDF_NOPROGRESSIVERENDERING;
 if dlopt_OfflineOperation in DownloadOptions then
  Result := Result or BINDF_OFFLINEOPERATION;
 if dlopt_GetNewestVersion in DownloadOptions then
  Result := Result or BINDF_GETNEWESTVERSION;
 if dlopt_NoWriteCache in DownloadOptions then
  Result := Result or BINDF_NOWRITECACHE;
 if dlopt_NeedFile in DownloadOptions then
  Result := Result or BINDF_NEEDFILE;
 if dlopt_PullData in DownloadOptions then
  Result := Result or BINDF_PULLDATA;
 if dlopt_IgnoreSecurityProblem in DownloadOptions then
  Result := Result or BINDF_IGNORESECURITYPROBLEM;
 if dlopt_Resynchronize in DownloadOptions then
  Result := Result or BINDF_RESYNCHRONIZE;
 if dlopt_HyperLink in DownloadOptions then
  Result := Result or BINDF_HYPERLINK;
 if dlopt_NoUI in DownloadOptions then
  Result := Result or BINDF_NO_UI;
 if dlopt_SilentOperation in DownloadOptions then
  Result := Result or BINDF_SILENTOPERATION;
 if dlopt_PragmaNoCache in DownloadOptions then
  Result := Result or BINDF_PRAGMA_NO_CACHE;
 if dlopt_DirectRead in DownloadOptions then
  Result := Result or BINDF_DIRECT_READ;
 if dlopt_FormsSubmit in DownloadOptions then
  Result := Result or BINDF_FORMS_SUBMIT;
 if dlopt_GetFromCacheIfNetFailed in DownloadOptions then
  Result := Result or BINDF_GETFROMCACHE_IF_NET_FAIL;
end;

{ TURL_Loader }

constructor TURL_Loader.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 if (AOwner <> nil) and (AOwner is TActiveForm) then
  FAXControl := (AOwner as TActiveForm).ActiveFormControl;
end;

destructor TURL_Loader.Destroy;
begin
 FURLDownloader.Free;
 inherited Destroy;
end;

procedure TURL_Loader.AbortDownload;
begin
 if FURLDownloader <> nil then
  FURLDownloader.AbortDownload;
end;

procedure TURL_Loader.BytesAvailableNotify(Sender: TURLDownloader;
  BytesAvailable: integer; var BytesToRead: integer;  var ReadBuffer: pointer);
begin
 if Assigned(FOnBytesAvailable) then
  FOnBytesAvailable(Self, BytesAvailable, BytesToRead, ReadBuffer);
end;

procedure TURL_Loader.DataReadNotify(Sender: TURLDownloader; Data: pointer;
  DataSize: integer);
begin
 if Assigned(FOnDataRead) then
  FOnDataRead(Self, Data, DataSize);
end;

procedure TURL_Loader.DownloadProgressNotify(Sender: TURLDownloader;
  const Progress: TDownloadProgress);
begin
 if Assigned(FOnDownloadProgress) then
  FOnDownloadProgress(Self, Progress);
end;

procedure TURL_Loader.DownloadStopNotify(Sender: TURLDownloader;
  ErrorCode: cardinal);
begin
 if Assigned(FOnDownloadStop) then
  FOnDownloadStop(Self, ErrorCode);
end;

procedure TURL_Loader.DownloadToStreamNotify(Sender: TDownloaderToStream;
  ErrorCode: cardinal; Stream: TStream; StartPosition: integer);
begin
 if Assigned(FOnDownloadToStream) then
  FOnDownloadToStream(Self, ErrorCode, Stream, StartPosition);
end;

function TURL_Loader.GetDownloadInProgress: boolean;
begin
 Result := (FURLDownloader <> nil) and FURLDownloader.DownloadInProgress;
end;

procedure TURL_Loader.HTTPResponseNotify(Sender: TURLDownloader;
  ResponseCode: DWord; const ResponseHeaders: string);
begin
 if Assigned(FOnHTTPResponse) then
  FOnHTTPResponse(Self, ResponseCode, ResponseHeaders);
end;

procedure TURL_Loader.SetURL(const Value: string);
begin
 if not DownloadInProgress then FURL := Value;
end;

procedure TURL_Loader.StartDownload;
begin
 StartDownloadCommon(false, nil);
end;

procedure TURL_Loader.StartDownloadCommon(ToStream: boolean; Stream: TStream);
var HTTPRequestData: THTTPRequestData;
begin
 if DownloadInProgress then exit;
 if Assigned(FOnHTTPRequestData) then
  FOnHTTPRequestData(Self, HTTPRequestData.Method, HTTPRequestData.ExtraHeaders,
                     HTTPRequestData.RequestBody);
 FURLDownloader.Free;
 FURLDownloader := nil;
 if ToStream then
  FURLDownloader := TDownloaderToStream.Create(Stream)
 else
  FURLDownloader := TURLDownloader.Create;
 FURLDownloader.BindFlags := DownloadOptionsToBindFlags(FDownloadOptions);
 FURLDownloader.OnDownloadProgress := DownloadProgressNotify;
 FURLDownloader.OnHTTPResponse := HTTPResponseNotify;
 if ToStream then begin
  TDownloaderToStream(FURLDownloader).OnDownloadToStream := DownloadToStreamNotify;
 end
 else begin
  FURLDownloader.OnBytesAvailable := BytesAvailableNotify;
  FURLDownloader.OnDataRead := DataReadNotify;
  FURLDownloader.OnDownloadStop := DownloadStopNotify;
 end;
 if FAXControl = nil then
  FURLDownloader.StartDownload( nil, FURL, @HTTPRequestData)
 else
  FURLDownloader.StartDownload( FAXControl as IUnknown, FURL, @HTTPRequestData);
end;

procedure TURL_Loader.StartDownloadToStream(Stream: TStream);
begin
 StartDownloadCommon(true, Stream);
end;

function TURL_Loader.GetTotalRead: integer;
begin
 if FURLDownloader <> nil then Result := FURLDownloader.TotalRead
 else Result := 0;
end;


end.
